@@ -1,9 +1,8 @@
-/*	$Id$ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
  * Copyright (c) 2014, Xavier Mendez, Devin Torres and the Hoedown authors
- * Copyright (c) 2016--2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -50,6 +49,10 @@ struct 	nroff {
 	struct bnodeq		**foots; /* footnotes */
 	size_t			  footsz; /* footnote size */
 	size_t			  indent; /* indentation width */
+	const char		 *cr; /* fixed-width font */
+	const char		 *cb; /* fixed-width bold font */
+	const char		 *ci; /* fixed-width italic font */
+	const char		 *cbi; /* fixed-width bold-italic font */
 };
 
 enum	bscope {
@@ -73,6 +76,7 @@ struct	bnode {
 	char				*args; /* (unsafe) 2nd args */
 	int				 close; /* BNODE_COLOUR/FONT */
 	int				 tblhack; /* BSCOPE_SPAN */
+        int				 headerhack; /* BSCOPE_BLOCK */
 	enum bscope			 scope; /* scope */
 	unsigned int			 font; /* if BNODE_FONT */
 #define	BFONT_ITALIC			 0x01
@@ -205,41 +209,47 @@ nstate_colour_buf(unsigned int ft)
  * For compatibility with traditional troff, return non-block font code
  * using the correct sequence of \fX, \f(xx, and \f[xxx].
  */
-static const char *
-nstate_font_buf(unsigned int ft, int blk)
+static int
+nstate_font(const struct nroff *st, struct lowdown_buf *ob,
+    unsigned int ft, int enclose)
 {
-	static char 	 fonts[10];
+	const char	*font = NULL;
+	char		 fonts[3];
 	char		*cp = fonts;
-	size_t		 len = 0;
+	size_t		 sz;
 
-	if (ft & BFONT_FIXED)
-		len++;
-	if (ft & BFONT_BOLD)
-		len++;
-	if (ft & BFONT_ITALIC)
-		len++;
-	if (ft == 0)
-		len++;
+	if (ft & BFONT_FIXED) {
+		if ((ft & BFONT_BOLD) && (ft & BFONT_ITALIC))
+			font = st->cbi;
+		else if (ft & BFONT_BOLD)
+			font = st->cb;
+		else if (ft & BFONT_ITALIC)
+			font = st->ci;
+		else
+			font = st->cr;
+	} else {
+		font = cp = fonts;
+		if (ft & BFONT_BOLD)
+			(*cp++) = 'B';
+		if (ft & BFONT_ITALIC)
+			(*cp++) = 'I';
+		if (ft == 0)
+			(*cp++) = 'R';
+		*cp = '\0';
+	}
 
-	if (!blk && len == 3)
-		(*cp++) = '[';
-	else if (!blk && len == 2)
-		(*cp++) = '(';
+	assert(font != NULL);
+	sz = strlen(font);
+	assert(sz > 0);
 
-	if (ft & BFONT_FIXED)
-		(*cp++) = 'C';
-	if (ft & BFONT_BOLD)
-		(*cp++) = 'B';
-	if (ft & BFONT_ITALIC)
-		(*cp++) = 'I';
-	if (ft == 0)
-		(*cp++) = 'R';
+	if (!enclose || sz == 1)
+		return hbuf_puts(ob, font);
 
-	if (!blk && len == 3)
-		(*cp++) = ']';
+	if (sz > 2)
+		return HBUF_PUTSL(ob, "[") &&
+		    hbuf_puts(ob, font) && HBUF_PUTSL(ob, "]");
 
-	(*cp++) = '\0';
-	return fonts;
+	return HBUF_PUTSL(ob, "(") && hbuf_puts(ob, font);
 }
 
 static int
@@ -354,11 +364,13 @@ bqueue_strip_paras(struct bnodeq *bq)
 }
 
 static int
-bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq, int esc)
+bqueue_flush(const struct nroff *st, struct lowdown_buf *ob,
+    const struct bnodeq *bq, int esc)
 {
 	const struct bnode	*bn, *chk, *next;
 	const char		*cp;
 	int		 	 nextblk;
+	char			 trailingchar;
 
 	TAILQ_FOREACH(bn, bq, entries) {
 		nextblk = 0;
@@ -411,12 +423,14 @@ bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq, int esc)
 		/* Print font and colour escapes. */
 
 		if (bn->scope == BSCOPE_FONT && nextblk) {
-			if (!hbuf_printf(ob, ".ft %s",
-			    nstate_font_buf(bn->font, nextblk)))
+			if (!HBUF_PUTSL(ob, ".ft "))
+				return 0;
+			if (!nstate_font(st, ob, bn->font, 0))
 				return 0;
 		} else if (bn->scope == BSCOPE_FONT) {
-			if (!hbuf_printf(ob, "\\f%s", 
-			    nstate_font_buf(bn->font, nextblk)))
+			if (!HBUF_PUTSL(ob, "\\f"))
+				return 0;
+			if (!nstate_font(st, ob, bn->font, 1))
 				return 0;
 		} else if (bn->scope == BSCOPE_COLOUR) {
 			assert(nextblk);
@@ -503,11 +517,21 @@ bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq, int esc)
 				return 0;
 		}
 
-		/* Finally, trailing newline. */
+		/*
+		 * The "headerhack" is used by SH block macros to
+		 * indicate that their children are all normal text or
+		 * entities, and so to output the content on the same
+		 * line as the SH macro.  This is a hack because man(7)
+		 * specifically allows for next-line content; however,
+		 * buggy software (e.g., Mac OS X's makewhatis) don't
+		 * correctly parse empty SH properly.
+		 */
+
+		trailingchar = bn->headerhack ? ' ' : '\n';
 
 		if (nextblk && ob->size > 0 && 
-		    ob->data[ob->size - 1] != '\n' &&
-		    !hbuf_putc(ob, '\n'))
+		    ob->data[ob->size - 1] != trailingchar &&
+		    !hbuf_putc(ob, trailingchar))
 			return 0;
 	}
 
@@ -649,7 +673,7 @@ putlink(struct bnodeq *obq, struct nroff *st,
 		goto out;
 	if (bq == NULL && !hbuf_putb(ob, link))
 		goto out;
-	else if (bq != NULL && !bqueue_flush(ob, bq, 1))
+	else if (bq != NULL && !bqueue_flush(st, ob, bq, 1))
 		goto out;
 
 	/*
@@ -828,6 +852,7 @@ rndr_header(struct nroff *st, struct bnodeq *obq,
 	struct lowdown_buf		*buf = NULL;
 	const struct lowdown_buf	*nbuf;
 	int			 	 rc = 0;
+        const struct lowdown_node	*child;
 
 	level = (ssize_t)n->rndr_header.level + st->headers_offs;
 	if (level < 1)
@@ -845,6 +870,25 @@ rndr_header(struct nroff *st, struct bnodeq *obq,
 			bqueue_block(obq, ".SS");
 		if (bn == NULL)
 			return 0;
+
+		/*
+		 * Do a scan of the contents of the SH.  If there's only
+		 * normal text (or entities), then record this fact.  It
+		 * will be used in bqueue_flush() for how the macro is
+		 * serialised.
+		 */
+
+		if (level == 1) {
+			bn->headerhack = 1;
+			TAILQ_FOREACH(child, &n->children, entries) {
+				if (child->type == LOWDOWN_ENTITY ||
+				    child->type == LOWDOWN_NORMAL_TEXT)
+					continue;
+				bn->headerhack = 0;
+				break;
+			}
+		}
+
 		TAILQ_CONCAT(obq, bq, entries);
 		st->post_para = 1;
 		return 1;
@@ -1288,13 +1332,23 @@ rndr_table_cell(struct bnodeq *obq, struct bnodeq *bq,
 }
 
 static int
-rndr_superscript(struct bnodeq *obq, struct bnodeq *bq)
+rndr_superscript(struct bnodeq *obq, struct bnodeq *bq,
+    enum lowdown_rndrt type)
 {
+	const char	*p1, *p2;
 
-	if (bqueue_span(obq, "\\u\\s-3") == NULL)
+	/* Lift pandoc's way of doing this. */
+
+	p1 = (type == LOWDOWN_SUPERSCRIPT) ?
+		"\\v\'-0.3m\'\\s[\\n[.s]*9u/12u]" : 
+		"\\v\'0.3m\'\\s[\\n[.s]*9u/12u]";
+	p2 = (type == LOWDOWN_SUPERSCRIPT) ?
+		"\\s0\\v\'0.3m\'" :
+		"\\s0\\v\'-0.3m\'";
+	if (bqueue_span(obq, p1) == NULL)
 		return 0;
 	TAILQ_CONCAT(obq, bq, entries);
-	return bqueue_span(obq, "\\s+3\\d") != NULL;
+	return bqueue_span(obq, p2) != NULL;
 }
 
 static int
@@ -1525,7 +1579,7 @@ rndr_meta(struct nroff *st, const struct bnodeq *bq,
 
 	if ((ob = hbuf_new(32)) == NULL)
 		return 0;
-	if (!bqueue_flush(ob, bq, 1)) {
+	if (!bqueue_flush(st, ob, bq, 1)) {
 		hbuf_free(ob);
 		return 0;
 	}
@@ -1843,8 +1897,11 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 	case LOWDOWN_LINK:
 		rc = rndr_link(st, obq, &tmpbq, &n->rndr_link);
 		break;
+	case LOWDOWN_SUBSCRIPT:
+		rc = rndr_superscript(obq, &tmpbq, n->type);
+		break;
 	case LOWDOWN_SUPERSCRIPT:
-		rc = rndr_superscript(obq, &tmpbq);
+		rc = rndr_superscript(obq, &tmpbq, n->type);
 		break;
 	case LOWDOWN_FOOTNOTE:
 		rc = rndr_footnote_ref(st, obq, &tmpbq);
@@ -1852,7 +1909,7 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		 * Restore what subsequent paragraphs should do.  This
 		 * macro will create output that's delayed in being
 		 * shown.  It might set post_para, which we don't want
-		 * to propogate to the actual output that will follow.
+		 * to propagate to the actual output that will follow.
 		 */
 		st->post_para = post_para;
 		break;
@@ -1929,7 +1986,7 @@ lowdown_nroff_rndr(struct lowdown_buf *ob,
 	st->post_para = 0;
 
 	if (rndr(&metaq, st, n, &bq)) {
-		if (!bqueue_flush(ob, &bq, 1))
+		if (!bqueue_flush(st, ob, &bq, 1))
 			goto out;
 		if (ob->size && ob->data[ob->size - 1] != '\n' &&
 		    !hbuf_putc(ob, '\n'))
@@ -1962,6 +2019,26 @@ lowdown_nroff_new(const struct lowdown_opts *opts)
 
 	p->flags = opts != NULL ? opts->oflags : 0;
 	p->man = opts != NULL && opts->type == LOWDOWN_MAN;
+	p->cr = opts != NULL ? opts->nroff.cr : NULL;
+	p->cb = opts != NULL ? opts->nroff.cb : NULL;
+	p->ci = opts != NULL ? opts->nroff.ci : NULL;
+	p->cbi = opts != NULL ? opts->nroff.cbi : NULL;
+
+	/*
+	 * Set the default "constant width" fonts.  This is complicated
+	 * because the "C" fixed-with font is not universally available
+	 * on all output media.  For example, -Tascii does not carry a
+	 * "C" font.  However, this is a good enough default.
+	 */
+
+	if (p->cr == NULL)
+		p->cr = "CR";
+	if (p->cb == NULL)
+		p->cb = "CB";
+	if (p->ci == NULL)
+		p->ci = "CI";
+	if (p->cbi == NULL)
+		p->cbi = "CBI";
 
 	/*
 	 * Set a default indentation.  For -man, we use 3 because we
