@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -323,6 +323,12 @@ odt_sty_flush(struct lowdown_buf *ob,
 				return 0;
 		}
 		break;
+	case LOWDOWN_SUBSCRIPT:
+		if (!HBUF_PUTSL(ob,
+		    "<style:text-properties"
+		    " style:text-position=\"sub 58%\"/>\n"))
+			return 0;
+		break;
 	case LOWDOWN_SUPERSCRIPT:
 		if (!HBUF_PUTSL(ob,
 		    "<style:text-properties"
@@ -957,8 +963,6 @@ odt_metaq_flush(struct lowdown_buf *ob,
 
 	/* Overrides. */
 
-	if (title == NULL)
-		title = "Untitled article";
 	if (rcsdate != NULL)
 		date = rcsdate;
 	if (rcsauthor != NULL)
@@ -967,12 +971,14 @@ odt_metaq_flush(struct lowdown_buf *ob,
 	if (!HBUF_PUTSL(ob, "<office:meta>\n"))
 		return 0;
 
-	if (!HBUF_PUTSL(ob, "<dc:title>"))
-		return 0;
-	if (!hesc_html(ob, title, strlen(title), 1, 0, 1))
-		return 0;
-	if (!HBUF_PUTSL(ob, "</dc:title>\n"))
-		return 0;
+	if (title != NULL) {
+		if (!HBUF_PUTSL(ob, "<dc:title>"))
+			return 0;
+		if (!hesc_html(ob, title, strlen(title), 1, 0, 1))
+			return 0;
+		if (!HBUF_PUTSL(ob, "</dc:title>\n"))
+			return 0;
+	}
 
 	if (author != NULL) {
 		if (!HBUF_PUTSL(ob, "<dc:creator>"))
@@ -1960,7 +1966,7 @@ rndr(struct lowdown_buf *ob,
 	int32_t				 ent;
 	struct odt			*st = ref;
 	struct odt_sty			*sty = NULL;
-	size_t				 curid = (size_t)-1, curoffs,
+	size_t				 curid = (size_t)-1, curoffs = 0,
 					 chngid = (size_t)-1;
 	int				 ret = 1;
 	void				*pp;
@@ -2111,6 +2117,7 @@ rndr(struct lowdown_buf *ob,
 	case LOWDOWN_EMPHASIS:
 	case LOWDOWN_STRIKETHROUGH:
 	case LOWDOWN_HIGHLIGHT:
+	case LOWDOWN_SUBSCRIPT:
 	case LOWDOWN_SUPERSCRIPT:
 		if (!rndr_span(ob, tmp, n, st))
 			goto out;
@@ -2225,7 +2232,7 @@ lowdown_odt_new(const struct lowdown_opts *opts)
 		return NULL;
 
 	p->flags = opts == NULL ? 0 : opts->oflags;
-	if (opts->odt.sty != NULL &&
+	if (opts != NULL && opts->odt.sty != NULL &&
 	    (p->sty = strdup(opts->odt.sty)) == NULL) {
 		free(p);
 		p = NULL;
